package ch.brotzilla.performance;

import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;

import ch.brotzilla.performance.tests.PerfTest;
import ch.brotzilla.performance.util.Utils;

public class TestRunner {

    private static final DecimalFormat f = new DecimalFormat("###,##0.00");
    
    private final List<TestThread> threads = new ArrayList<>();
    
    public TestRunner(TestType testType, int numberOfThreads) {
        for (int thread = 0; thread < numberOfThreads; thread++) {
            threads.add(new TestThread(testType.createPerfTest(Utils.createRandomTriangles(), thread)));
        }
    }
    
    public List<PerfTest> execute() throws InterruptedException {
        for (final TestThread thread : threads) {
            thread.start();
        }
        final List<PerfTest> result = new ArrayList<>();
        for (final TestThread thread : threads) {
            thread.join();
            result.add(thread.getPerfTest());
        }
        return result;
    }
    
    public static void saveImages(TestType type, List<PerfTest> tests) throws IOException {
        if (!StaticSettings.OutputFolder.exists())
            return;
        for (final PerfTest test : tests) {
            ImageIO.write(test.getImage(), "PNG", new File(StaticSettings.OutputFolder, type.getName() + "_" + test.getThread() + ".png"));
        }
    }
    
    public static void runTests(TestType... tests) throws InterruptedException, IOException {
        System.out.println("Running the following tests:");
        final StringBuffer buffer = new StringBuffer();
        for (final TestType testType : tests) {
            if (buffer.length() > 0)
                buffer.append(", ");
            buffer.append(testType.getName());
        }
        System.out.println("    " + buffer.toString());
        System.out.println();
        
        for (final TestType type : tests) {
            System.out.println("Running test '" + type.getName() + "'...");
            final TestRunner testRunner = new TestRunner(type, StaticSettings.NumberOfThreads);
            final List<PerfTest> results = testRunner.execute();
            double totalTrianglesPerSec = 0.0;
            for (final PerfTest result : results) {
                final double timeInMS = result.getTimeInMS();
                final double trianglesPerSec = Utils.computeTrianglesPerSec(timeInMS);
                totalTrianglesPerSec += trianglesPerSec;
                System.out.println("    Thread " + result.getThread() + ": " + f.format(trianglesPerSec) + " triangles/sec (" + f.format(timeInMS) + " ms)");
            }
            System.out.println("    Total   : " + f.format(totalTrianglesPerSec) + " triangles/sec");
            System.out.println();
            saveImages(type, results);
        }
    }
    
    public static void main(String[] args) throws InterruptedException, IOException {
        
        runTests(TestType.JoglFixedFunctions, TestType.Java2DVolatileImage, TestType.Java2DBufferedImage);
        
    }
    
}
