package ch.brotzilla.performance.settings;

import java.util.Properties;

public abstract class AbstractSettings {
    
    public AbstractSettings() {
        loadDefaults();
    }
    
    public AbstractSettings(String prefix, Properties properties) {
        loadProperties(prefix, properties);
    }
    
    public void loadProperties(String prefix, Properties properties) {
        _loadProperties(prefix, properties);
        validate();
    }
    
    public void loadDefaults() {
        _loadDefaults();
        validate();
    }
    
    public abstract void validate();

    protected abstract void _loadProperties(String prefix, Properties properties);

    protected abstract void _loadDefaults();
    
    protected String fullName(String prefix, String key) {
        return (prefix != null && !prefix.trim().isEmpty() ? prefix.trim() + "." : "") + key;
    }
    
    protected String loadString(String prefix, String key, Properties properties, String defaultValue) {
        final String value = properties.getProperty(fullName(prefix, key), null);
        return (value != null && !value.trim().isEmpty()) ? value.trim() : defaultValue;
    }
    
    protected int loadInteger(String prefix, String key, Properties properties, int defaultValue) {
        final String value = loadString(prefix, key, properties, null);
        return (value != null) ? Integer.parseInt(value) : defaultValue;
    }
    
    protected boolean loadBoolean(String prefix, String key, Properties properties, boolean defaultValue) {
        final String value = loadString(prefix, key, properties, null);
        if (value != null) {
            if (value.toLowerCase().equals("true"))
                return true;
            if (value.toLowerCase().equals("false"))
                return false;
            throw new IllegalArgumentException("Invalid boolean value: " + value);
        }
        return defaultValue;
    }
    
    protected void checkNotNull(Object obj, String key) {
        if (obj == null)
            throw new NullPointerException("The property '" + key + "' must not be null");
    }
    
    protected void checkNotEmpty(String str, String key) {
        if (str == null || str.trim().isEmpty())
            throw new IllegalArgumentException("The property '" + key + "' must not be empty");
    }
    
    protected void checkGreaterThanZero(int value, String key) {
        if (value <= 0)
            throw new IllegalArgumentException("The property '" + key + "' has to be greater than zero");
    }
    
}
