package ch.brotzilla.performance.tests;

import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;

import com.jogamp.opengl.DefaultGLCapabilitiesChooser;
import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLDrawableFactory;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.util.GLReadBufferUtil;

import ch.brotzilla.performance.StaticSettings;
import ch.brotzilla.performance.util.Triangle;
import ch.brotzilla.performance.util.Triangles;
import ch.brotzilla.performance.util.Utils;

public class PerfTestJogl extends PerfTest {

    private final BufferedImage image;
    private final int[] pixelData;
    private final IntBuffer pixelBuffer;
    
    private GLAutoDrawable drawable;
    private GL2 gl;
    private GLReadBufferUtil pixelReader;

    public PerfTestJogl(Triangles triangles, int thread) {
        super(triangles, thread);
        image = new BufferedImage(StaticSettings.Width, StaticSettings.Height, BufferedImage.TYPE_INT_ARGB);
        pixelData = ((DataBufferInt) image.getRaster().getDataBuffer()).getData();
        pixelBuffer = IntBuffer.wrap(pixelData);
    }

    @Override
    public BufferedImage getImage() {
        return image;
    }
    
    @Override
    protected void initialize() {
        drawable = createDrawable();
        gl = drawable.getGL().getGL2();
        pixelReader = new GLReadBufferUtil(true, false);
        setup(gl);
    }
    
    @Override
    protected void render() {
        renderTriangles(gl);
        readPixels(gl, pixelReader);
    }

    @Override
    protected long computePixelSum() {
        return Utils.computePixelSum(pixelData);
    }

    private GLAutoDrawable createDrawable() {
        final GLProfile glp = GLProfile.get(GLProfile.GL2);
        final GLCapabilities caps = createCapabilities(glp);
        final GLDrawableFactory factory = GLDrawableFactory.getFactory(glp);
        final GLAutoDrawable drawable = factory.createOffscreenAutoDrawable(factory.getDefaultDevice(), caps, new DefaultGLCapabilitiesChooser(), StaticSettings.Width, StaticSettings.Height);
        drawable.display();
        drawable.getContext().makeCurrent();
        return drawable;
    }
    
    private GLCapabilities createCapabilities(GLProfile glp) {
        final GLCapabilities caps = new GLCapabilities(glp);
        caps.setHardwareAccelerated(true);
        caps.setDoubleBuffered(false);
        caps.setOnscreen(false);
        caps.setRedBits(8);
        caps.setBlueBits(8);
        caps.setGreenBits(8);
        if (StaticSettings.UseTranslucency) {
            caps.setAlphaBits(8);
        } else {
            caps.setAlphaBits(0);            
        }
//        if (StaticSettings.UseAntialiasing) {
//            caps.setSampleBuffers(true);
//            caps.setNumSamples(2);
//            caps.setAccumRedBits(16);
//            caps.setAccumGreenBits(16);
//            caps.setAccumBlueBits(16);
//            caps.setAccumAlphaBits(16);
//        }
        return caps;
    }
    
    private void setup(GL2 gl) {
        gl.glMatrixMode(GL2.GL_PROJECTION);
        gl.glLoadIdentity();
        gl.glOrtho(0, StaticSettings.Width, 0, StaticSettings.Height, -1, 1);
        gl.glMatrixMode(GL2.GL_MODELVIEW);
        gl.glLoadIdentity();
        gl.glViewport(0, 0, StaticSettings.Width, StaticSettings.Height);
        gl.glDepthMask(false);
        gl.glDisable(GL.GL_DEPTH_TEST);
        if (StaticSettings.UseAlphaBlending) {
            gl.glEnable(GL.GL_BLEND);
            if (StaticSettings.UseTranslucency) {
            	StaticSettings.BlendFunc.apply(gl);
            } else {
            	gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
            }
        } else {
        	gl.glDisable(GL.GL_BLEND);
        }
        if (StaticSettings.UseAntialiasing) {
            gl.glShadeModel(GL2.GL_SMOOTH);
            gl.glEnable(GL2.GL_POLYGON_SMOOTH);
            gl.glHint(GL2.GL_POLYGON_SMOOTH_HINT, GL.GL_NICEST);
        }
    }

    private void renderTriangle(GL2 gl, Triangle triangle) {
        if (StaticSettings.UseAlphaBlending) {
        	gl.glColor4f(triangle.b, triangle.g, triangle.r, triangle.a);
        	gl.glVertex2d(triangle.ax, triangle.ay);
        	gl.glColor4f(triangle.b, triangle.g, triangle.r, triangle.a);
        	gl.glVertex2d(triangle.bx, triangle.by);
        	gl.glColor4f(triangle.b, triangle.g, triangle.r, triangle.a);
        	gl.glVertex2d(triangle.cx, triangle.cy);
        } else {
        	gl.glColor3f(triangle.b, triangle.g, triangle.r);
        	gl.glVertex2d(triangle.ax, triangle.ay);
        	gl.glColor3f(triangle.b, triangle.g, triangle.r);
        	gl.glVertex2d(triangle.bx, triangle.by);
        	gl.glColor3f(triangle.b, triangle.g, triangle.r);
        	gl.glVertex2d(triangle.cx, triangle.cy);
        }
    }
    
    private void renderTriangles(GL2 gl) {
    	if (StaticSettings.UseTranslucency) {
    		StaticSettings.ColorMask.apply(gl);
    		final float[] color = StaticSettings.TranslucentBackgroundColor.getComponents(null);
    		gl.glClearColor(color[0], color[1], color[2], color[3]);
    	} else {
    		gl.glColorMask(true, true, true, true);
    		final float[] color = StaticSettings.OpaqueBackgroundColor.getComponents(null);
    		gl.glClearColor(color[0], color[1], color[2], color[3]);
    	}
         gl.glClear(GL.GL_COLOR_BUFFER_BIT);
        gl.glColorMask(true, true, true, true);
        gl.glBegin(GL.GL_TRIANGLES);
        for (final Triangle triangle : getTriangles()) {
            renderTriangle(gl, triangle);
        }
        gl.glEnd();
    }

    private void readPixels(GL2 gl, GLReadBufferUtil pixelReader) {
        pixelReader.readPixels(gl, false);
        final IntBuffer pixelBuf = ((ByteBuffer) pixelReader.getPixelBuffer().buffer).asIntBuffer();
        pixelBuffer.put(pixelBuf);
        pixelBuffer.rewind();
    }

}